import { ContainerModule, interfaces, Container } from 'inversify';
import {
    ClientTypes,
    IGeneratedSettings as ISettings,
    IContextKeyService,
    IEditorService,
    IModelService,
    IModeService,
    ICodeEditorService,
    IClientConfigurationService as IConfigurationService,
    ILifecycleService,
    IPreferenceEditorService,
} from '@workbench-stack/core';
import { CodeEditorService, ModelService, ModeService, ResourceContextKey, ContextKeyService, ConfigurationService,
    LifecycleService,
} from '../services';

export const workbenchModule: () => interfaces.ContainerModule =
    () => new ContainerModule((bind: interfaces.Bind) => {
        bind<ICodeEditorService>(ClientTypes.ICodeEditorService).to(CodeEditorService).inSingletonScope();
        bind<IModelService>(ClientTypes.IModelService).to(ModelService).inSingletonScope();
        bind<IModeService>(ClientTypes.IModeService).to(ModeService).inSingletonScope();
        bind<IContextKeyService>(ClientTypes.IContextKeyService).to(ContextKeyService).inSingletonScope();

        bind<ResourceContextKey>(ClientTypes.IResourceContextKey).to(ResourceContextKey).inSingletonScope();

        bind<IConfigurationService>(ClientTypes.IConfigurationService).to(ConfigurationService).inSingletonScope();

        bind<ILifecycleService>(ClientTypes.ILifecycleService).to(LifecycleService).inSingletonScope();


        bind<interfaces.Container>('Container').toDynamicValue((context: interfaces.Context) => {
            const childContainer = new Container();
            childContainer.parent = context.container;
            return childContainer;
        });
    });
