/* tslint:disable */
// const esmRequire = require('esm')(module)
// const part= esmRequire('./editor-groups-helper.ts')
import * as part from './editor-groups-helper';
import { IEditorGroup } from '@workbench-stack/core';
import "jest";

describe("Workbench editor groups", () => {

    it('openEditors / closeEditors', function () {
        let editorGroups: IEditorGroup[] = [];
        const groupdId = 1;
        const group = part.getOrCreateGroup(1, editorGroups);
        expect(group.id).toBe(groupdId);
        editorGroups = part.updateGroup(group, editorGroups);

        // look for the group again
        const group1 = part.getOrCreateGroup(1, editorGroups);
        expect(group1.id).toBe(groupdId);
        editorGroups = part.updateGroup(group, editorGroups);
        expect(editorGroups.length).toBe(1);

        // close group
        editorGroups = part.removeGroup(1, editorGroups);
        // should not remove the root group
        expect(editorGroups.length).toBe(1);

        // add another group
        const group2 = part.getOrCreateGroup(2, editorGroups);
        editorGroups = part.updateGroup(group2, editorGroups);
        expect(group2.id).toBe(2);
        editorGroups = part.removeGroup(1, editorGroups);
        expect(editorGroups.length).toBe(1);
        expect(editorGroups[0].id).toBe(2)

        // last element in the group won't get deleted
        editorGroups = part.removeGroup(2, editorGroups);
        expect(editorGroups.length).toBe(1);
        expect(editorGroups[0].id).toBe(2)
    })

    it('update editor', function () {
        let editorGroups: IEditorGroup[] = [];
        const groupdId = 1;
        const group = part.getOrCreateGroup(1, editorGroups, { activate: true });
        expect(group.id).toBe(groupdId);
        editorGroups = part.updateGroup(group, editorGroups);

        // add another group
        const group2 = part.getOrCreateGroup(2, editorGroups, { activate: true });
        editorGroups = part.updateGroup(group2, editorGroups);
        expect(group2.id).toBe(2);

    })

    it('activate editor', function () {
        let editorGroups: IEditorGroup[] = [];
        const groupdId = 1;
        const group = part.getOrCreateGroup(1, editorGroups, { activate: true });
        expect(group.id).toBe(groupdId);
        editorGroups = part.updateGroup(group, editorGroups);
        const group2 = part.getOrCreateGroup(2, editorGroups, { activate: true});
        editorGroups = part.updateGroup(group2, editorGroups);
        expect(group2.id).toBe(2);

        editorGroups = part.activateGroup(group.id, editorGroups);
        console.log('--groups', editorGroups);
        editorGroups = part.activateGroup(group2.id, editorGroups);
        console.log('--groups2', editorGroups);
        editorGroups = part.activateGroup(group.id, editorGroups);
        console.log('--groups2', editorGroups);
    })
});
