import {
  IViewsRegistry,
  Extensions as ViewContainerExtensions,
  IViewContainersRegistry,
  IViewContainer,
} from './views-registry';
import 'jest';
import * as React from 'react';
import { IViewDescriptor } from '@workbench-stack/core';
import { Registry } from '@vscode/monaco-editor/esm/vs/platform/registry/common/platform';
import { MockedProvider } from '@apollo/react-testing';
import * as ReactTestRenderer from 'react-test-renderer';
import { useQuery } from '@apollo/react-hooks';
import gql from 'graphql-tag';
const wait = require('waait');

const container = Registry.as<IViewContainersRegistry>(
  ViewContainerExtensions.ViewContainersRegistry,
).registerViewContainer('test');
const ViewsRegistry = Registry.as<IViewsRegistry>(
  ViewContainerExtensions.ViewsRegistry,
);


// test cases similar to
// https://www.apollographql.com/docs/react/development-testing/testing/
export const GET_VIEWS = gql`
  query getViews($id: String) {
    getViews(id: $id) {
      id
      name
    }
  }
`;

export const DATA_PROVIDER = gql`
    query GetTreeData($viewId: String ) {
        getTreeData(viewId: $viewId) {
            label
            description
            icon
            resourceUri
            contextValue
            children {
                label
                description
                icon
                resourceUri
                contextValue
            }
        }
    }

`;


export const TREE_REFRESH = gql`
    mutation refreshTreeData()
`;



describe('ContributableViewModel', () => {
  function ViewContainer({ id }) {
    const { loading, error, data } = useQuery(GET_VIEWS, { variables: { id } });
    if (loading) {
      return <p>Loading...</p>;
    }
    if (error) {
      return <p>Error!</p>;
    }
    console.log('ID', id, 'data', data);
    return (
        <p>
            {data.getViews.map(e => e.id + ' with name ' + e.name)}
        </p>
    );
  }

function View({id, name }) {
      const { loading, error, data } = useQuery(DATA_PROVIDER, {
        variables: { viewId: id },
      });

      if (loading) {
          return <p>Loading...</p>;
      }
      if (error) {
          return <p>Error!</p>;
      }

      return (

      );
}
  it('register/unregister', async () => {
    const viewDescriptor: IViewDescriptor = {
      id: 'view1',
      name: 'Test View 1',
    };

    const view1: IViewDescriptor = { id: 'view1', name: 'Test View 1' };
    const view2: IViewDescriptor = { id: 'view2', name: 'Test View 2' };
    ViewsRegistry.registerViews([view1, view2], container);

    console.log('----', container, ViewsRegistry.getViews(container));
    // const key = ContextKeyService.createKey()
    const mocks = [
        {
          request: {
            query: GET_VIEWS,
            variables: {
              id: 'test',
            },
          },
          result: {
            data: {getViews: ViewsRegistry.getViews(container)},
          },
        },
        {
            request: {
              query: DATA_PROVIDER,
              variables: {
                viewId: 'test',
              },
            },
            result: {
                data: {getTreeData: ViewsRegistry.getViews(container)},
            },
          },
      ];
    const component = ReactTestRenderer.create(
      <MockedProvider mocks={mocks} addTypename={false}>
        <ViewContainer id="test" />
      </MockedProvider>
    );
    await wait(0); // wait for response
    const tree = component.toJSON();
    console.log(tree);
  });
});
