import { Feature, FeatureWithRouterFactory } from '@common-stack/client-react';
import { interfaces } from 'inversify';
import LayoutModule from '@workbench-stack/layout';
import ModalModule from '@workbench-stack/modal';
import { ThemeModule } from '@workbench-stack/components';
import SettingsModule from '@workbench-stack/settings';
import OutputModule from '@workbench-stack/output-browser';
import { resolvers, schema, defaults, fragmentMatcher, dataIdFromObject } from './graphql';
import { workbenchModule } from './containers';
import {
    IEditorService,
    ClientTypes,
    IModelService,
    IClientService,
    IModeService,
    IContextKeyService,
    ICodeEditorService,
    IResourceContextKey,
    IEditorGroupsService,
    ILifecycleService,
} from '@workbench-stack/core';

const workbenchServiceGen = (container: interfaces.Container): Pick<IClientService, Exclude<keyof IClientService, 'textFileService' | 'untitledEditorService' | 'preferenceEditorService'>> => {
  return {
    editorService: container.get<IEditorService>(ClientTypes.IEditorService),
    editorGroupsService: container.get<IEditorGroupsService>(ClientTypes.IEditorGroupsService),
    codeEditorService: container.get<ICodeEditorService>(ClientTypes.ICodeEditorService),
    modelService: container.get<IModelService>(ClientTypes.IModelService),
    modeService: container.get<IModeService>(ClientTypes.IModeService),
    contextKeyService: container.get<IContextKeyService>(ClientTypes.IContextKeyService),
    resourceContextKey: container.get<IResourceContextKey>(ClientTypes.IResourceContextKey),
    lifecycleService: container.get<ILifecycleService>(ClientTypes.ILifecycleService),
  };
};


export default new Feature(
  FeatureWithRouterFactory,
  LayoutModule, ThemeModule,
  ModalModule, SettingsModule,
  OutputModule,
  new Feature({
    clientStateParams: [{ resolvers, typeDefs: [schema], defaults, fragmentMatcher: <any>fragmentMatcher }],
    createContainerFunc: workbenchModule,
    createServiceFunc: workbenchServiceGen,
    dataIdFromObject,
  }),
);
