import { BehaviorSubject, from, combineLatest} from 'rxjs';
import { TestScheduler } from 'rxjs/testing';
import { first, map, take, tap } from 'rxjs/operators';
import { IEditorService, IModelService, ICommonEditor, EditorActionType } from '@workbench-stack/extension-api';
import { ITextModel } from '@vscode/monaco-editor/esm/vs/editor/common/model';
import { ICodeEditor } from '@vscode/monaco-editor/esm/vs/editor/browser/editorBrowser';
import { URI } from '@vscode/monaco-editor/esm/vs/base/common/uri';
import * as monaco from '@vscode/monaco-editor/esm/vs/editor/editor.api';
import { createEditorService } from './editor-service';
import 'jest';
import { createModelService } from './model-service';
const scheduler = () => new TestScheduler((a, b) => expect(a).toEqual(b));


describe('EditorService', () => {
    describe('editors', () => {

    });
    document.body.innerHTML =
        '<div>' +
        '  <div id="test" />' +
        '</div>';

    xtest('addEditor', async () => {

        const editorService = createEditorService();
        const modelUri = monaco.Uri.parse('a://b/foo.json'); // a made up unique URI for our model
        const model = monaco.editor.createModel('', 'json', modelUri);

        // Create the actual Monaco editor.
        const editor = monaco.editor.create(document.getElementById('test'), {
            lineNumbers: 'on',
            automaticLayout: true,
            minimap: { enabled: false },
            formatOnType: true,
            formatOnPaste: true,
            autoIndent: true,
            renderIndentGuides: false,
            glyphMargin: false,
            folding: false,
            renderLineHighlight: 'none',
            scrollBeyondLastLine: false,
            quickSuggestions: true,
            quickSuggestionsDelay: 200,
            wordBasedSuggestions: false,
            wordWrap: 'on',
            theme: 'vs-dark',
            model,
        });
        const editor2 = monaco.editor.create(document.getElementById('test'), {
            lineNumbers: 'on',
            automaticLayout: true,
            minimap: { enabled: false },
            formatOnType: true,
            formatOnPaste: true,
            autoIndent: true,
            renderIndentGuides: false,
            glyphMargin: false,
            folding: false,
            model,
        });

        editorService.addEditor(editor as any);
        editorService.addEditor(editor2 as any);
        // editorService.addEditor(editor as any);

        await from(editorService.action$)
            .pipe(
                map(el => el.editors.length),
                tap(e => { console.log('>>>', e) }),
                // first(),
                take(3),
            ).toPromise();
    });

    xtest('activeEditor', async () => {
        const editorService = createEditorService();
        const modelUri = monaco.Uri.parse('a://b/foo.json'); // a made up unique URI for our model
        const model = monaco.editor.createModel('', 'json', modelUri);

        // Create the actual Monaco editor.
        const editor = monaco.editor.create(document.getElementById('test'), {
            lineNumbers: 'on',
            automaticLayout: true,
            minimap: { enabled: false },
            formatOnType: true,
            formatOnPaste: true,
            autoIndent: true,
            renderIndentGuides: false,
            glyphMargin: false,
            folding: false,
            renderLineHighlight: 'none',
            scrollBeyondLastLine: false,
            quickSuggestions: true,
            quickSuggestionsDelay: 200,
            wordBasedSuggestions: false,
            wordWrap: 'on',
            theme: 'vs-dark',
            model,
        });
        const editor2 = monaco.editor.create(document.getElementById('test'), {
            lineNumbers: 'on',
            automaticLayout: true,
            minimap: { enabled: false },
            formatOnType: true,
            formatOnPaste: true,
            autoIndent: true,
            renderIndentGuides: false,
            glyphMargin: false,
            folding: false,
            model,
        });
        editorService.addEditor(editor as any);
        editorService.addEditor(editor2 as any);
        // editorService.removeEditor(test.getId());

        editorService.changeActiveEditor(editor.getId());

        await from(editorService.action$)
            .pipe(
                map(el => el.editors.length),
                tap(e => { console.log('>>>', e) }),
                first(),
            ).toPromise();
    });


});
